
import plac
import psycopg2
from pathlib import Path
import spacy
from joblib import Parallel, delayed
from functools import partial
from spacy.util import minibatch
import time
import multiprocessing_on_dill as multiprocessing


@plac.annotations(
    output_dir=("Output directory", "positional", None, Path),
    model=("Model name (needs tagger)", "positional", None, str),
    n_jobs=("Number of workers", "option", "n", int),
    batch_size=("Batch-size for each process", "option", "b", int),
)
def main(output_dir="/tmp/", model="en_core_web_sm", n_jobs=16, batch_size=35000):
    rundates = ['2019-12-10','2019-12-11','2019-12-12','2019-12-13','2019-12-14','2019-12-15','2019-12-16']


    if not output_dir.exists():
        output_dir.mkdir()

    nlp = spacy.load(model)  # load spaCy model
    all_stopwords = nlp.Defaults.stop_words
    all_stopwords.add('rt')

    for rundate in rundates:
        print("processing " + str(rundate))
        process_file(rundate, nlp, n_jobs, batch_size, output_dir, all_stopwords)    


    #for rundate in rundates:



def process_file(rundate, nlp, n_jobs, batch_size, output_dir, all_stopwords):
    start = time.time()
    input_data = query(rundate)
    partitions = minibatch(input_data, size=batch_size)
    executor = Parallel(n_jobs=n_jobs, backend="multiprocessing", prefer="processes")
    do = delayed(partial(transform_texts, nlp))
    tasks = (do(i, batch, output_dir, all_stopwords, rundate) for i, batch in enumerate(partitions))
    executor(tasks)

    print('time taken for file ' + str(rundate) + ' ---> ' + str(time.time() - start))



def transform_texts(nlp, batch_id, input_data, output_dir, all_stopwords, rundate):
    print(nlp.pipe_names)
    outfile = open(str(output_dir) + '/' + str(batch_id) + '_' + rundate, 'w')
    ids, texts = zip(*input_data)
    ids_list = list(ids)
    print("Processing batch", batch_id)
    index = 0
    for doc in nlp.pipe(texts, disable = ['ner']):
        chunks = list(doc.noun_chunks)
        tokens_without_sw= [word for word in chunks if not word.text.lower() in all_stopwords]
        listToStr = ' '.join([str(elem) for elem in tokens_without_sw])
        outfile.write(str(ids_list[index]) + '|' + listToStr)
        outfile.write("\n")
        index = index + 1

    print("Saved {} texts to {}.txt".format(len(texts), batch_id))
    outfile.close


def query(rundate):
    conn = None
    try:
        conn = psycopg2.connect("host=capstone.cvrqe3hdombn.us-east-1.rds.amazonaws.com dbname=postgres user=postgres password=password")
        cur = conn.cursor()
        cur.execute("select tweet_id, text from tweet where date(created) = '" + rundate + "'")
        rows = cur.fetchall()
        cur.close()
        return rows
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            conn.close()


if __name__ == "__main__":
    plac.call(main)
