from database import query
import urllib.request
from urllib.error import HTTPError
import time
import numpy as np
import cv2
import os
import sys

IMAGE_PATH = '/Users/Vamsi/Downloads/ti/'
IMAGE_HEIGHT = 224
IMAGE_WIDTH = 224
HORIZONTAL_WIDTH = 50

def get_image_urls_from_db():
    try:
        return query("SELECT distinct url, url_hash FROM MEDIA")
    except:
        print("error connecting to database")
        return

def get_full_image_path(img):
    return IMAGE_PATH + 'res_' + img + '.jpg'

        
def download_and_resize_images(image_tuples):
    #print("Method called - processor : " + str(process) + " with " + str(len(image_tuples)) + " images")
    for idx,image in enumerate(image_tuples):
        try:
            time.sleep(0.005)
            IMAGE_LOCAL_PATH = IMAGE_PATH + image[1] + '.jpg'
            RESIZE_IMAGE_LOCAL_PATH =  IMAGE_PATH + 'res_' + image[1] + '.jpg'
            urllib.request.urlretrieve(image[0], IMAGE_LOCAL_PATH)
            input_img=cv2.imread(IMAGE_LOCAL_PATH, cv2.IMREAD_UNCHANGED)
            im = cv2.resize(input_img,(IMAGE_HEIGHT,IMAGE_WIDTH))
            cv2.imwrite(RESIZE_IMAGE_LOCAL_PATH, im)
            os.remove(IMAGE_LOCAL_PATH)
            print("Process file - " + str(idx))
        except HTTPError:
            print("HTTP Error for " + image[0])
        except:
            print("Unknown error",  sys.exc_info()[0])

def combine_images_in_cluster(cluster_idx, images):
    print("Got " + str(len(images)) + " images.")
    horizontal_blocks = []
    vertical_blocks = []
    for idx, img in enumerate(images):
        horizontal_blocks.append(cv2.imread(img))
        if (idx+1) % HORIZONTAL_WIDTH == 0:
            vertical_blocks.append(np.concatenate(tuple(horizontal_blocks), axis=1))
            horizontal_blocks = []

    if len(horizontal_blocks) > 0:
        blank_image = np.zeros(shape=[IMAGE_WIDTH, IMAGE_WIDTH, 3], dtype=np.uint8)
        for i in range(HORIZONTAL_WIDTH - len(horizontal_blocks)):
            horizontal_blocks.append(blank_image)
        vertical_blocks.append(np.concatenate(tuple(horizontal_blocks), axis=1))
    output = np.concatenate(tuple(vertical_blocks), axis = 0)
    cv2.imwrite(IMAGE_PATH + 'cluster-' + str(cluster_idx) + '.jpg', output)
    

    # img1 = cv2.imread(IMAGE_PATH + 'res_img01c911ed6e20333d4cfc6ccdfc68cc58.jpg')
    # img2 = cv2.imread(IMAGE_PATH + 'res_img2bfde62ab2b1b65b38bc71250cb914b3.jpg')
    # vis = np.concatenate((img1, img2), axis=1)
    # cv2.imwrite(IMAGE_PATH + 'out.jpg', vis)


def main(): 
    image_tuples = get_image_urls_from_db()
    download_and_resize_images(image_tuples)

if __name__ == '__main__':
    images = []
    images.append(IMAGE_PATH + 'res_img01c911ed6e20333d4cfc6ccdfc68cc58.jpg')
    images.append(IMAGE_PATH + 'res_img2bfde62ab2b1b65b38bc71250cb914b3.jpg')
    images.append(IMAGE_PATH + 'res_img2bfde62ab2b1b65b38bc71250cb914b3.jpg')
    images.append(IMAGE_PATH + 'res_img2bfde62ab2b1b65b38bc71250cb914b3.jpg')
    images.append(IMAGE_PATH + 'res_img2bfde62ab2b1b65b38bc71250cb914b3.jpg')
    images.append(IMAGE_PATH + 'res_img2bfde62ab2b1b65b38bc71250cb914b3.jpg')
    images.append(IMAGE_PATH + 'res_img2bfde62ab2b1b65b38bc71250cb914b3.jpg')
    images.append(IMAGE_PATH + 'res_img2bfde62ab2b1b65b38bc71250cb914b3.jpg')
    combine_images_in_cluster(1, images)