import numpy as np
import time
import cv2
import os

YOLO_LOCATION = "/home/ec2-user/yolo/darknet/cfg"
CONFIDENCE = 0.5
THRESHOLD = 0.3
labelsPath = os.path("/home/ec2-user/yolo/darknet/data/coco.names")
LABELS = open(labelsPath).read().strip().split("\n")
weightsPath = os.path.sep.join([YOLO_LOCATION, "yolov3.weights"])
configPath = os.path.sep.join([YOLO_LOCATION, "yolov3.cfg"])
net = cv2.dnn.readNetFromDarknet(configPath, weightsPath)

def detect_objects_from_images(i):
    if not os.path.exists(i):
        print("Image {} doesn't exist.".format(i)) 
        return [False for i in range(len(LABELS))]
    image = cv2.imread(i)
    (H, W) = image.shape[:2]
    ln = net.getLayerNames()
    ln = [ln[i[0] - 1] for i in net.getUnconnectedOutLayers()]
    blob = cv2.dnn.blobFromImage(image, 1 / 255.0, (416, 416),swapRB=True, crop=False)
    net.setInput(blob)
    start = time.time()
    layerOutputs = net.forward(ln)
    end = time.time()
    print("[INFO] YOLO took {:.3f} seconds".format(end - start))

    confidences = []
    classIDs = []

    for output in layerOutputs:
        for detection in output:
            scores = detection[5:]
            classID = np.argmax(scores)
            confidence = scores[classID]
            if confidence > CONFIDENCE:
                confidences.append(float(confidence))
                classIDs.append(classID)
    output = [i in classIDs for i in range(len(LABELS))]
    return output



if __name__ == '__main__':
    print(detect_objects_from_images('/Users/Vamsi/Downloads/ti/res_img0f1d14475366e8eb554a5707c10cfec3.jpg'))
