import cv2 
import pytesseract 
import os
import pickle
import time
import numpy as np
import boto3
import psycopg2
import sys

YOLO_LOCATION = "/home/ec2-user/yolo/darknet/cfg"
CONFIDENCE = 0.5
THRESHOLD = 0.3
labelsPath = "/home/ec2-user/yolo/darknet/data/coco.names"
LABELS = open(labelsPath).read().strip().split("\n")
weightsPath = os.path.sep.join([YOLO_LOCATION, "yolov3.weights"])
configPath = os.path.sep.join([YOLO_LOCATION, "yolov3.cfg"])
net = cv2.dnn.readNetFromDarknet(configPath, weightsPath)

AWS_KEY_ID='AKIAYQWOE66CVTICVL4Z'
AWS_SECRET='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'
AWS_S3_BUCKET = 'dse-cohort5-group6'

s3_client = boto3.client('s3', aws_access_key_id=AWS_KEY_ID,aws_secret_access_key=AWS_SECRET)


def generate(table):
    sql = "SELECT column_name, data_type " \
            "FROM information_schema.columns " \
            "WHERE table_name = '%s'" % table
    #print(sql)
    conn = None
    try:
        conn = psycopg2.connect("host=postgres.cvrqe3hdombn.us-east-1.rds.amazonaws.com dbname=postgres user=postgres password=password")
        cur = conn.cursor()        
        cur.execute(sql)
        row = cur.fetchone()
        columns = []
        types = []

        while row is not None:
            columns.append(row[0])
            types.append("%s")
            row = cur.fetchone()
        cur.close()
        insert_sql = "INSERT INTO {} ({}) VALUES({})" \
                        .format(table, ','.join(columns), ','.join(types))
        #print(insert_sql)
        return insert_sql
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            conn.close()

def save(table, data):
    conn = None
    try:
        
        conn = psycopg2.connect("host=postgres.cvrqe3hdombn.us-east-1.rds.amazonaws.com dbname=postgres user=postgres password=password")
        cur = conn.cursor()    
        sql = generate(table)    
        cur.executemany(sql,data)
        conn.commit()
        cur.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            conn.close()


def detect_objects(image):
    start = time.time()
    (H, W) = image.shape[:2]
    ln = net.getLayerNames()
    ln = [ln[i[0] - 1] for i in net.getUnconnectedOutLayers()]
    blob = cv2.dnn.blobFromImage(image, 1 / 255.0, (416, 416),swapRB=True, crop=False)
    net.setInput(blob)
    start = time.time()
    layerOutputs = net.forward(ln)

    confidences = []
    classIDs = []

    for output in layerOutputs:
        for detection in output:
            scores = detection[5:]
            classID = np.argmax(scores)
            confidence = scores[classID]
            if confidence > CONFIDENCE:
                confidences.append(float(confidence))
                classIDs.append(classID)
    output = [i in classIDs for i in range(len(LABELS))]
    end = time.time()
    #print("Objects in image are : " + str(output))
    #print("Object Detection took {:.3f} seconds".format(end - start))

    return output

def read_text(img):
    start = time.time()
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY) 
    ret, thresh1 = cv2.threshold(gray, 0, 255, cv2.THRESH_OTSU | cv2.THRESH_BINARY_INV) 
    rect_kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (18, 18)) 
    dilation = cv2.dilate(thresh1, rect_kernel, iterations = 1) 
    _, contours, hierarchy = cv2.findContours(dilation, cv2.RETR_EXTERNAL,cv2.CHAIN_APPROX_NONE)
    im2 = img.copy() 
    text =''
    for cnt in contours: 
        x, y, w, h = cv2.boundingRect(cnt)         
        rect = cv2.rectangle(im2, (x, y), (x + w, y + h), (0, 255, 0), 2) 
        cropped = im2[y:y + h, x:x + w] 
        text += pytesseract.image_to_string(cropped) 
        text += ' '
    #print('Image Text is ' + text)
    #print("Text Extraction took {:.3f} seconds".format(time.time() - start))
    if text:
        return text.replace('\r', ' ').replace('\n', ' ')
    else:
        return ''

def process_text(counter):
    totstart = time.time()
    text_list = []
    obj_list = []
    start_index = counter * 100
    end_index = (counter + 1) * 100
    print("Start Index : " + str(start_index) + " End Index : " + str(end_index))
    for i in range(start_index, end_index):
        try :
            s3_client.download_file(AWS_S3_BUCKET, "images/" + str(i) + ".jpg", "/tmp/" + str(i) + ".jpg")
            img = cv2.imread("/tmp/" + str(i) + ".jpg") 
            image_text = read_text(img)
            objects = detect_objects(img)
            text_list.append((i, image_text))
            obj_list.append(tuple([i] + objects))
            os.remove("/tmp/" + str(i) + ".jpg")
        except:
            continue
            #print("Error " + str(i))
    save('image_text', text_list)
    save('image_features', obj_list)
    print("End Time {:.3f} seconds".format(time.time() - totstart))




c1 = int(sys.argv[1])
c2 = int(sys.argv[2])

for i in range(c1,c2):
    process_text(i)